﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using System.Reflection;

public class EasyMouseAndKeyboadVRControls : MonoBehaviour {

    [SerializeField]
    bool hideControlHints = false;
    Transform neck;
	Transform rightHand;
	Transform leftHand;



	[Tooltip("Key used to move the right controller forward")]
	public KeyCode extendLeftControllerKey = KeyCode.C;
	public KeyCode extendRightControllerKey = KeyCode.V;

	[Tooltip("How far forward the controllers move upon hitting the extend key")]
	public float controllerExtendDistance = 1;

	public float controllerExtendAmtRight =0;
	public float controllerExtendAmtLeft =0;



    // Use this for initialization
    void Start () {

        VRTK.SDK_InputSimulator vrtkInputSim = this.GetComponent<VRTK.SDK_InputSimulator>();

        //this.GetComponent<VRTK.SDK_InputSimulator>().
        //this.GetComponentInParent<VRTK.VRTK_SDKManager>().scriptAliasLeftController;
        leftHand = this.GetComponentInParent<VRTK.VRTK_SDKSetup>().actualLeftController.transform;
		rightHand = this.GetComponentInParent<VRTK.VRTK_SDKSetup>().actualRightController.transform;
		neck =  this.GetComponentInParent<VRTK.VRTK_SDKSetup>().actualHeadset.transform.parent;

            this.GetComponent<VRTK.SDK_InputSimulator>().triggerAlias = KeyCode.Mouse0;
            this.GetComponent<VRTK.SDK_InputSimulator>().gripAlias = KeyCode.Mouse1;


            var controllerSDK = VRTK.VRTK_SDK_Bridge.GetControllerSDK() as VRTK.SDK_SimController;
            if (controllerSDK != null)
            {
                Dictionary<string, KeyCode> keyMappings = new Dictionary<string, KeyCode>()
                {
                    {"Trigger", KeyCode.Mouse0 },
                    {"Grip", KeyCode.Mouse1 },

                    {"TouchpadPress", vrtkInputSim.touchpadAlias },
                    {"ButtonOne", vrtkInputSim.buttonOneAlias },
                    {"ButtonTwo", vrtkInputSim.buttonTwoAlias },
                    {"StartMenu", vrtkInputSim.startMenuAlias },
                    {"TouchModifier", vrtkInputSim.touchModifier },
                    {"HairTouchModifier", vrtkInputSim.hairTouchModifier }
                };
                controllerSDK.SetKeyMappings(keyMappings);
            }
        
    }
	
	void LateUpdate()
	{

		this.GetComponent<VRTK.SDK_InputSimulator> ().showControlHints = false;

		float targetRight = 0;
			if (Input.GetKey (extendRightControllerKey)) {
				targetRight = 1;
			}

			float targetLeft = 0;
			if (Input.GetKey (extendLeftControllerKey)) {
				targetLeft = 1;
			}

			controllerExtendAmtRight = Mathf.MoveTowards (controllerExtendAmtRight, targetRight, Time.deltaTime / .5f);

			controllerExtendAmtLeft = Mathf.MoveTowards (controllerExtendAmtLeft, targetLeft, Time.deltaTime / .5f);

			Vector3 localLook = Quaternion.Euler (neck.transform.eulerAngles.x, 0, 0) * Vector3.forward;

			rightHand.transform.localPosition = neck.localPosition + (localLook * (.4f + controllerExtendAmtRight)) + new Vector3 (.2f, 0, 0); //new Vector3(0.2f, 1.2f, 0.5f);
			rightHand.transform.localRotation = Quaternion.identity;

        rightHand.forward = neck.forward;

			leftHand.transform.localPosition = neck.localPosition + (localLook * (.4f + controllerExtendAmtLeft)) + new Vector3 (-.2f, 0, 0);  //new Vector3(-0.2f, 1.2f, 0.5f);
			leftHand.transform.localRotation = Quaternion.identity;

        leftHand.forward = neck.forward;
		
	}

    

	void OnGUI()
	{
        if (!hideControlHints)
        {
            VRTK.SDK_InputSimulator vrtkInputSim = this.GetComponent<VRTK.SDK_InputSimulator>();
            if (GUI.Button(new Rect(15, 30, 130, 35), "Hide Instructions"))
            {
                hideControlHints = true;
                vrtkInputSim.showControlHints = false;

            }
            GUIStyle style = new GUIStyle();
            style.fontSize = 16;
            style.fontStyle = FontStyle.Bold;



            style.normal.textColor = Color.black;

           

            Transform currentHand = (Transform)GetInstanceField(typeof(VRTK.SDK_InputSimulator), this.GetComponent<VRTK.SDK_InputSimulator>(), "currentHand");
            bool isHand = (bool)GetInstanceField(typeof(VRTK.SDK_InputSimulator), this.GetComponent<VRTK.SDK_InputSimulator>(), "isHand");
            string leftOrRight = currentHand == leftHand.transform ? "LEFT (red)" : "RIGHT (green)";

            string grabButton = vrtkInputSim.triggerAlias == KeyCode.Mouse1 ? "RIGHT-CLICK" : vrtkInputSim.triggerAlias == KeyCode.Mouse0 ? "LEFT-CLICK" : vrtkInputSim.triggerAlias.ToString();
            string grabMessage = "hold '" + grabButton + "' to grab objects\n"
                +
                "Grabbing only works with the " + leftOrRight + " controller";

            //"' to grab with " + (currentHand == leftHand.transform ? "LEFT" : "RIGHT") + " controller";




            string message =
                "'WASD' + Mouse: to Move/Look\n" +
                "'" + extendLeftControllerKey + "'" + " to extend left controller\n" +
                "'" + extendRightControllerKey + "'" + " to extend right controller\n" +

                "'" + this.GetComponent<VRTK.SDK_InputSimulator>().touchpadAlias + "' to teleport\n"


                + grabMessage;// this.GetComponent<VRTK.SDK_InputSimulator>()."";

            if (isHand)
            {
                message = "press 'ALT' or 'OPTION' to restore controls";
            }

            float yOffset = 45;
            GUI.Label(new Rect(15, yOffset+ 25, 500, 500),
                message, style);
            GUI.Label(new Rect(17, yOffset + 26, 500, 500),
                message, style);

            style.normal.textColor = Color.white;
            GUI.Label(new Rect(16, yOffset + 25, 500, 500),
                message, style);
        }
		
	}

    internal static object GetInstanceField(System.Type type, object instance, string fieldName)
    {
        BindingFlags bindFlags = BindingFlags.Instance | BindingFlags.Public | BindingFlags.NonPublic
            | BindingFlags.Static;
        FieldInfo field = type.GetField(fieldName, bindFlags);
        
        return field.GetValue(instance);
    }

}
